# API - Arquivos de Personalização
Adicionar, alterar, ativar ou inativar arquivos de personalização, além de obter as Licenças de Uso do cliente
## Produto requisitado:

| Nome                     | url            | uri                                      |
|--------------------------|----------------|------------------------------------------|
| Customer Portal | https://cp.teknisa.com | https://cp.teknisa.com/backend/index.php |

## Verbos HTTPs permitidos:
- POST

## Informações de acesso

A alteração ou adição de novos locais de uso podem ser feitos através da requisição de uma rotina do **CP**, contanto que os parâmetros corretos sejam enviados nos formatos adequados.
Para acessar as informações disponibilizadas na **API** é necessário realizar duas requisições HTTP.

A primeira requisição consiste em
obter o token de acesso ao **CP**. A segunda, já em posse do token, o
requisitante acessa as rotas disponibilizadas na **API**. As requisições devem ser feitas a partir do backend do produto requisitante. 

Para realizar as requisições HTTP é indicado o uso da **Lib**. O
arquivo [Utilities.php](http://gitlab.teknisa.com/teknisa/libbackend/blob/master/src/Teknisa/Libs/Util/Utilities.php) da Lib contém o método **_callWebService_** que é
responsável por realizar as chamadas HTTP utilizando a biblioteca **cURL**.

> <span style="color: red;">**Nota:**</span> Para os produtos que não fazem uso da Lib, é necessário replicar o metodo **_callWebService_**.


Método **_callWebService_**:

```php
public function callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url, $keepConnected = false)
{
    set_time_limit(180);
    $header = array();
    $header[] = 'OAuth-Token: ' . $token;
    $header[] = 'OAuth-SecretId: ' . $privateKey;
    if($keepConnected) {
        $header[] = 'OAuth-KeepConnected: ' . $keepConnected;
    }
    $header[] = 'Content-Type: application/json';
    $header[] = 'Content-Length: ' . strlen(json_encode($data));

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
    curl_setopt($ch, CURLOPT_URL, $url . $requestType);
    curl_setopt($ch, CURLOPT_POST, true);
    curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
    curl_setopt($ch, CURLOPT_TIMEOUT, 30);

    $result = curl_exec($ch);
    $result = json_decode($result, true);
    $error = curl_error($ch);
    curl_close($ch);

    if (!empty($result['error'])) {
        throw new \Exception($result['error']);
    } else if (!empty($error)) {
        throw new \Exception($error);
    } else {
        return isset($result['dataset'][$dataSetResultPos]) ? $result['dataset'][$dataSetResultPos] : null;
    }
}
```

**Parâmetros**

| parâmetro      | tipo   | descrição                                                                                                                          |
|----------------|--------|------------------------------------------------------------------------------------------------------------------------------------|
|  $token        | String | Token de acesso necessário para acessar rotas autenticadas. Podendo ser null quando a requisição for feita para requisitar o mesmo |
| $requestType   | String | A rota no qual o requisitante está tentando acessar                                                                                |
| $data          | Array  | Dados do corpo da requisição.                                                                                                      |
| $dataResultPos | String | Nome do dataset no qual o requisitante irá acessar os dados na resposta da requisição.                                             |
| $privateKey    | String | Chave de acesso (client secret) necessária para acessar o produto (CP)                                                             |
| $uri           | String | uri para qual vai ser feita a requisição HTTP (do Produto CP no caso)                                                              |

### Procedimento

Sempre será necessário realizar duas requisições HTTP para obter informações da API. Uma requisição para solicitar o token e a segunda para acessar os dados da API. O token terá um tempo de vida de **2 horas**, sendo assim, o mesmo token pode ser utilizado para realizar requisições que não ultrapassem esse tempo.

### Dados
Existem 3 rotinas para se acessar, cada uma com seus respectivos parâmetros.

- Linhas de produto disponíveis:
  - **NRORG** - Número da Organização.
    - _Atentar para ambientes locais, pois o NRORG preenchido na base é 1. Deve ser enviado o NRORG correto do cliente._

- Ativar/Inativar um arquivo de personalização existente:
  - **NRORG** - Número da Organização;
  - **CDLOCALUSO** - Código do local de uso;
  - **IS_ACTIVE** - Ativo ou não (S ou N).

- Adicionar ou alterar um arquivo de personalização (_os parâmetros devem ser passados com máscara, caso possuam_):
  - **CDLOCALUSO**: Código do local de Uso;
  - **NRORG**: Número da Organização;;
  - **IDLUSOATIV\***: Ativo (S ou N);
  - **NMFANTLUSO**: Nome Fantasia;
  - **NMRAZSOCLUSO**: Razão social;
  - **IDTPIJURLUSO**: Tipo de Organização:
    - **F**: Pessoa física;
    - **J**: Pessoa jurídica;
    - **I**: Inscrição internacional;
    - **O**: Outros.
  - **NRINSJURLUSO**: Número de Inscrição - deverá ser enviado com a máscara:
    - Pessoa física (XXX.XXX.XXX-XX);
    - Pessoa jurídica (XX.XXX.XXX/XXXX-XX).
  - **NRINSESTLUSO**: Inscrição Estadual;
  - **NRINSMUNILUSO**: Inscriçao Municipal;
  - **NMCONTLUSO**: Contato;
  - **NMDEPTOLUSO**: Departamento;
  - **DSENDELUSO**: Endereço (Rua);
  - **NMBAIRLUSO**: Endereço (Bairro);
  - **NRCEPLUSO**: CEP 
    - Deverá ser enviado com a máscara (XXXXX-XXX);
  - **CDPAIS**: País;
  - **SGESTADO**: Estado;
  - **CDMUNICIPIO**: Município;
  - **NRTELELUSO\***: Telefone;
  - **NRRAMALLUSO\***: Ramal;
  - **NRFAXLUSO\***: Fax;
  - **DSEMAILCOMLU**: Email;
  - **CDLINPCLIE\***: Código da linha de produto;
    - Consultado atráves do WebService _Linhas de produto disponíveis_.
  - **IDFATURAFILISEP\***: Faturamento separado por filial (_S/N_, por padrão _N_)
  - **IDMODPROC\***: Modo de processamento:
    - F: Filial;
    - M: Matriz;
    - Padrão: F.
  - **IDLOCAPROC\***:  Local de processamento:
    - F: Filial;
    - M: Matriz;
    - Padrão: F.

> <span style="color: red;">**Nota:**</span> Parâmetros com * são opcionais.

## Rotas

```
/lib_requestToken
```
Método `POST`

dataset response `token`

###### Exemplo de uso:

```php
$token = null; 
$requestType = '/lib_requestToken'; 
$data = array(
    'CLIENT_ID' => 'cd7e85178153by60953518945f9hakt19a50a4',
    'CLIENT_SECRET' => 'lib5449996c21f780fe30badd6521931a5sac9'
); 
$dataSetResultPos = 'token'; 
$privateKey = 'lib5449996c21f780fe30badd6521931a5sac9'; 
$url = 'https://cp.teknisa.com/backend/index.php';

$token = callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url);
```

### Retorno

```
array(1) {
  [0]=>
  string(38) "32746369a147f4e47ceb2305cf55c3c20a9582"
}
```
<br/>
<hr/>
<br/>

```
/api/findUseLicense
```
Método `POST`

request type `FilterData`

FilterData `NRORG` 

dataset response `findUseLicense`

###### Exemplo de uso:

```php
$token = $token[0];  // token obtido através da rota 'lib_requestToken'
$requestType = '/api/findUseLicense';
$nrorg = '20';
$data = array(
    'requestType' => 'FilterData',
    'filter' => array(
        array('name' => 'NRORG', 'operator' => '=', 'value' => $nrorg),
    )
); 
$dataSetResultPos = 'findUseLicense'; 
$privateKey = 'lib5449996c21f780fe30badd6521931a5sac9'; 
$url = 'https://cp.teknisa.com/backend/index.php'; 

$result = callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url);
```

### Retorno

```
array(39) {
   [0]=>
  array(8) {
    ["CDLINPCLIE"]=>
    string(20) "00000000000000001142"
    ["NMLINPCLIE"]=>
    string(8) "TekNutri"
    ["IDMPROCLINPC"]=>
    string(1) "F"
    ["IDLPROCLINPC"]=>
    string(1) "F"
    ["NRUSUALINPC"]=>
    string(1) "1"
    ["NRACESLINPC"]=>
    string(1) "0"
    ["IDTIPLINPC"]=>
    string(1) "M"
    ["IDMLPROCLINPC"]=>
    string(3) "F/F"
  }
  ...
```

<br/>
<hr/>
<br/>

```
/api/findCountry
```
Método `POST`

request type `FilterData`

FilterData `NRORG` `CDPAIS` `NMPAIS`

dataset response `findCountry`

###### Exemplo de uso:

```php
$token = $token[0];  // token obtido através da rota 'lib_requestToken'
$requestType = '/api/findCountry';
$nrorg = '20';
$data = array(
    'requestType' => 'FilterData',
    'filter' => array(
        array('name' => 'NRORG', 'operator' => '=', 'value' => $nrorg),
    )
); 
$dataSetResultPos = 'findCountry'; 
$privateKey = 'lib5449996c21f780fe30badd6521931a5sac9'; 
$url = 'https://cp.teknisa.com/backend/index.php'; 

$result = callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url);
```

### Retorno

```
array(56) {
  [0]=>
  array(5) {
    ["CDPAIS"]=>
    string(4) "0055"
    ["NMPAIS"]=>
    string(6) "Brasil"
    ["NRORG"]=>
    string(2) "20"
    ["CDESOCIAL"]=>
    NULL
    ["ORDPAIS"]=>
    string(1) "1"
  }
  ...
```

<br/>
<hr/>
<br/>

```
/api/findState
```
Método `POST`

request type `FilterData`

FilterData `CDPAIS` `SGESTADO` `NMESTADO`

dataset response `findState`

<span style="color: red;">**Nota:**</span> O filtro CDPAIS é obrigatório.

###### Exemplo de uso:

```php
$token = $token[0];  // token obtido através da rota 'lib_requestToken'
$requestType = '/api/findState';
$cdpais = '0055';
$data = array(
    'requestType' => 'FilterData',
    'filter' => array(
        array('name' => 'CDPAIS', 'operator' => '=', 'value' => $cdpais),
    )
); 
$dataSetResultPos = 'findState'; 
$privateKey = 'lib5449996c21f780fe30badd6521931a5sac9'; 
$url = 'https://cp.teknisa.com/backend/index.php'; 

$result = callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url);
```

### Retorno

```
array(28) {
  [0]=>
  array(3) {
    ["SGESTADO"]=>
    string(2) "AC"
    ["NMESTADO"]=>
    string(4) "ACRE"
    ["CDPAIS"]=>
    string(4) "0055"
  }
  ...
```

<br/>
<hr/>
<br/>

```
/api/findCity
```
Método `POST`

request type `FilterData`

FilterData `CDPAIS` `SGESTADO` `CDMUNICIPIO` `NMMUNICIPIO`

dataset response `findCity`

<span style="color: red;">**Nota:**</span> Os filtros CDPAIS e SGESTADO são obrigatórios.

###### Exemplo de uso:

```php
$token = $token[0];  // token obtido através da rota 'lib_requestToken'
$requestType = '/api/findCity';
$cdpais = '0055';
$sgestado = 'PA';
$data = array(
    'requestType' => 'FilterData',
    'filter' => array(
        array('name' => 'CDPAIS', 'operator' => '=', 'value' => $cdpais),
        array('name' => 'SGESTADO', 'operator' => '=', 'value' => $sgestado),
    )
);
$dataSetResultPos = 'findCity';
$privateKey = 'lib5449996c21f780fe30badd6521931a5sac9'; 
$url = 'https://cp.teknisa.com/backend/index.php'; 

$result = callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url);
```

### Retorno

```
array(137) {
  [0]=>
  array(4) {
    ["CDMUNICIPIO"]=>
    string(4) "0401"
    ["NMMUNICIPIO"]=>
    string(10) "ABAETETUBA"
    ["SGESTADO"]=>
    string(2) "PA"
    ["CDPAIS"]=>
    string(4) "0055"
  }
  ...
```
<br/>
<hr/>
<br/>

```
/api/saveCustomizationFiles
```
Método `POST`

request type `DataSet `

response `saveCustomizationFiles`

###### Exemplo de uso:

```php
$token = $token[0];  // token obtido através da rota 'lib_requestToken'
$requestType = '/api/saveCustomizationFiles'; 
$data = array(
  "requestType" => "DataSet",
  "dataset" => array( array(
    'NRORG' => '20',
    'CDLOCALUSO' => 'AAAA',
    'NMFANTLUSO' => 'Teknisa',
    'NMRAZSOCLUSO' => 'Teknisa Software',
    'IDTPIJURLUSO' => 'J',
    'NRINSJURLUSO' => '00.000.000/0000-00',
    'NRINSESTLUSO' => '00',
    'NRINSMUNILUSO' => '00',
    'NMCONTLUSO' => 'Funcionario X',
    'NMDEPTOLUSO' => 'Departamento Y',
    'DSENDELUSO' => 'Rua W',
    'NMBAIRLUSO' => 'Bairro Y',
    'NRCEPLUSO' => '00000-000',
    'CDPAIS' => '0055',
    'SGESTADO' => 'MG',
    'CDMUNICIPIO' => '4123',
    'DSEMAILCOMLU' => 'funcionario@teknisa.com',
    'CDLINPCLIE' => ["00000000000000005297"],
    'IDMODPROC' => 'F',
    'IDLOCAPROC' => 'F'
  ))
); 
$dataSetResultPos = 'saveCustomizationFiles'; 
$privateKey = 'lib5449996c21f780fe30badd6521931a5sac9'; 
$url = 'https://cp.teknisa.com/backend/index.php'; 

$result = callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url);
```

### Retorno

```
array(1) {
  [0]=>
  array(1) {
    ["CDLOCALUSO"]=>
    string(4) "Teste"
  }
}
```
<br/>
<hr/>
<br/>

```
/api/customizationFilesActivation
```
Método `POST`

request type `DataSet`

dataset response `customizationFilesActivation`

###### Exemplo de uso:

```php
$token = $token[0];  // token obtido através da rota 'lib_requestToken'
$requestType = '/api/customizationFilesActivation'; 
$data = array(
  "requestType" => "DataSet",
  "dataset" => array( array(
    'CDLOCALUSO' => 'AAAA',
    'NRORG' => '20',
    'IS_ACTIVE' => 'S'
  ))
); 
$dataSetResultPos = 'customizationFilesActivation'; 
$privateKey = 'lib5449996c21f780fe30badd6521931a5sac9'; 
$url = 'https://cp.teknisa.com/backend/index.php';

$result = callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url);
```

### Retorno

```
array(1) {
  [0]=>
  bool(true)
}
```
<br/>
<hr/>
<br/>
