var ZeedhiDiretiveDate = angular.module("ZeedhiDiretiveDate", []);

(function() {

	function daysInMonth(iMonth, iYear) {
		return 32 - new Date(iYear, iMonth, 32).getDate();
	}

	function DateCtrl($scope, element, attrs, controller) {
		var dayField = $('.selectDay', element);
		var monthField = $('.selectMonth', element);
		var yearField = $('.selectYear', element);

		this.init = function() {
			this.getDateValue();

			this.loadYears(element);

			dayField.change(this.apply.bind(this));

			monthField.change(this.apply.bind(this));

			yearField.change(this.apply.bind(this));
		};

		this.getDateValue = function() {
			var day = dayField.val();
			var month = monthField.val();
			var year = yearField.val();

			var maxDayForMonth = daysInMonth(month - 1, year);

			$scope.__maxDay__ = maxDayForMonth;

			day = day > maxDayForMonth ? maxDayForMonth : day;

			return day + "/" + month + "/" + year;
		};

		this.loadYears = function() {
			var initialized = element.children().attr("data-future");
			if (!initialized) {
				var newestYearElement = angular.element("[data-newest]", element);
				var newestYear = newestYearElement.val();
				var thisYear = new Date().getFullYear();
				var options = [];
				while (newestYear < (thisYear + 20)) {
					var option = $("<option></option>");
					option.val(++newestYear);
					option.text(newestYear);
					option.data("future");
					options.unshift(option);
				}
				newestYearElement.parent().prepend(options);
			}
		};

		this.apply = function() {
			var value = this.getDateValue();
			var _apply = function() {
				controller.$setViewValue(value);
			};
			if (!($scope.$$phase == '$apply' || $scope.$$phase == '$digest') && !$scope.applyInProgressZh) {
				$scope.$apply(_apply);
			} else {
				_apply();
			}
			element.change();
		};

		this.change = function(value) {
			if (value) {
				var dateValues = value.split('/');
				dayField.val(dateValues[0]);
				monthField.val(dateValues[1]);
				yearField.val(dateValues[2]);
			}
			return value;
		};

	}

	ZeedhiDiretiveDate.directive('zhDateField', function() {
		return {
			require: '?ngModel',
			link: function($scope, element, attrs, controller) {

				var dateCtrl = new DateCtrl($scope, element, attrs, controller);

				dateCtrl.init();

				controller.$parsers.unshift(dateCtrl.change.bind(dateCtrl));

				controller.$formatters.unshift(dateCtrl.change.bind(dateCtrl));

			}
		};
	});
	ZeedhiDiretiveDate.directive("zhDateTime", function(ZhUtil, ScreenService) {
		/*
		 *   Date Time diective for wheel selector. Defaults to a date + time scroller.
		 *   Accepts the parameters described in opt. Consult docs to see what values they accept.
		 */
		return {
			restrict: 'A',
			link: function(scope, element, attrs, ctrl) {
				var field = scope.field || scope.question;
				// @todo parse pt_br to pt-BR, because moment and datetime use i18n with differents formats..
				var scopeLang = ZhUtil.formatLangToMobiscroll(scope.lang);

				function fixMobiscrollLang(language){
					if(language.length > 2){
						if(language.substring(0,2).toLowerCase() === 'es'){
							language = 'es';
						} else {
							language = language.substring(0,2).toLowerCase() + '-' + language.substring(3,5).toUpperCase();
						}
					}
					return language;
				}

				function beforeShowPicker(scrollerInstance) {
					change();
					scrollerInstance.setValue(element.val());
				}

				function change() {
					var maxDate = ZhUtil.evaluate(field['datetime-maxDate'] || '2100-01-01 23:59');
					var minDate = ZhUtil.evaluate(field['datetime-minDate'] || '1900-01-01 00:00');
					var defaultDate = ZhUtil.evaluate(field['datetime-defaultDate'] || "today");
					var lang = field['datetime-lang'] || scopeLang;
					lang = lang ? (lang === 'en-US' ? '' : fixMobiscrollLang(lang)) : 'pt-BR';

					defaultDate = defaultDate === "today" ? moment() : moment(defaultDate);
					maxDate = maxDate === "today" ? moment() : moment(new Date(maxDate));
					minDate = minDate === "today" ? moment() : moment(new Date(minDate));

					var opt = {
						preset: field['datetime-preset'] || 'datetime',
						dateOrder: field['datetime-dateOrder'],
						dateFormat: field['datetime-dateFormat'],
						timeWheels: field['datetime-timeWheels'],
						timeFormat: field['datetime-timeFormat'],
						defaultValue: defaultDate._d,
						minDate: minDate._d,
						maxDate: maxDate._d,
						stepMinute: field['datetime-stepMinute'] || 1,
						theme: 'default',
						mode: 'scroller',
						lang: lang,
						hourText: "Hours",
						minuteText: "Minutes",
						secText: "Seconds",
						display: field['datetime-display'] || 'bottom',
						animate: field['datetime-animate'] || '',
						headerText: ScreenService.i18n(field.label) || '',
						onBeforeShow: beforeShowPicker
					};

					if(!field.isReadOnly()){
						element.mobiscroll(opt);
					}
				}

				element.focus(change);
				angular.element(document).ready(change);
				scope.$watch('field', change);

				scope.clearField = function(field){
					if (!field.isReadOnly()) {
						field.widget.currentRow[field.name] = "";

						if(field.change){
							field.change();
						}
					}
				};
			}
		};
	});

	ZeedhiDiretiveDate.directive("zhBetweenDates", function(ZhUtil, ScreenService, templateManager) {
		return {
			restrict: 'A',
			link: function($scope, element, attrs, ctrl) {
				var field = $scope.field;
				var startField = element.find("input:first");
				var endField = element.find("input:last");
				var errorObj = {
					message: field.invalidIntervalMessage || ScreenService.i18n('The first date must be lower than second.')
				};

				var addErrorMessage = function(field) {
					if (!field.widget.currentRow.$error) {
						field.widget.currentRow.$error = {};
					}
					if (!field.widget.currentRow.$error[field.name]) {
						field.widget.currentRow.$error[field.name] = [];
					}
					var notExists = true;
					for (var i in field.widget.currentRow.$error[field.name]) {
						if (field.widget.currentRow.$error[field.name][i] === errorObj) {
							notExists = false;
							break;
						}
					}
					if (notExists) {
						field.widget.currentRow.$error[field.name].push(errorObj);
					}
				};

				var removeErrorMessage = function(field) {
					if (field.widget.currentRow.$error && field.widget.currentRow.$error[field.name]) {
						for (var i in field.widget.currentRow.$error[field.name]) {
							if (field.widget.currentRow.$error[field.name][i] === errorObj) {
								field.widget.currentRow.$error[field.name].splice(i, 1);
							}
						}
					}
				};

				var change = function(e) {
					if (this.blur) {
						this.blur();
					}

					var rowValue = {
						start: startField.val(),
						end: endField.val()
					};

					if (field.widget.currentRow) {
						field.widget.currentRow[field.name] = rowValue;

						var startDate = rowValue.start.split('/').reverse().join('-');
						var endDate = rowValue.end.split('/').reverse().join('-');

						if (moment(new Date(startDate)).isAfter(new Date(endDate))) {
							addErrorMessage(field);
						} else {
							removeErrorMessage(field);
						}

						if (field.change) {
							field.change();
						}
						templateManager.updateTemplate();
					}
				};

				var changeValueInUi = function(rowValue) {
					if (rowValue && rowValue.start) {
						startField.val(rowValue.start);
					}

					if (rowValue && rowValue.end) {
						endField.val(rowValue.end);
					}
				};

				startField.focus(change);
				endField.focus(change);
				$scope.$watch('field.widget.currentRow.' + field.name, changeValueInUi);
			}
		};
	});

	ZeedhiDiretiveDate.directive("zhDateType", function(templateManager, $timeout) {
		return {
			restrict: 'A',
			link: function($scope, $element){

				var BACKSPACE = 8;
				var DELETE = 46;
				var CONTROL = 17;
				var ARROW_UP = 38;
				var ARROW_LEFT = 37;

				var LEAP_YEAR = moment("1992", "YYYY");

				var REGEXP_FORMAT = {
					DD: /^[0][1-9]|^[1-2][0-9]|^[3][0-1]|^[0-3][_]|^[_][0-9]|[_]{2}/g,
					MM: /^[0][1-9]|^[1][0-2]|^[0-1][_]|^[_][0-9]|[_]{2}/g,
					YY: /[0-9_]{2}/g,
					YYYY: /[0-9_]{4}/g
				};

				var lang = $scope.field['datetime-lang'] || templateManager.lang || window.lang;
				var defaultFormat = moment().locale(lang).localeData().longDateFormat("L");
				var dateFormat = $scope.dateFormat || $scope.field.dateFormat || $scope.widget.dateFormat || defaultFormat;
				var displayFormat = $scope.displayFormat || $scope.field.displayFormat || $scope.widget.displayFormat || dateFormat;
				var separator = displayFormat.match(/\W/)[0];
				var formats = displayFormat.split(separator);

				var dateCompleted = $scope.dateCompleted || function(){};

				function valueIsFullfilled(value){
					return value.indexOf('_') === -1;
				}

				function setCursorPosition(pos){
					pos = pos < 0 ? 0 : pos;
					$element[0].setSelectionRange(pos, pos);
				}

				function getRegExpForFormat(format){
					return REGEXP_FORMAT[format.toUpperCase()];
				}

				function getSelectionStart(){
					return $element[0].selectionStart;
				}

				function getPos(){
					return $element[0].pos;
				}

				function getMaxDayOfMonth(month, year, yearFormat){
					var date;

					if(!month || !valueIsFullfilled(month)){
						month = "01";
					}
					if(!year || !valueIsFullfilled(year)){
						year = LEAP_YEAR.format(yearFormat);
					}

					date = moment(month + year, "MM"+yearFormat);
					return date.endOf("month").format("DD");
				}

				function getYearFormat(formats){
					var format =  formats.indexOf("YYYY") !== -1 ?
					formats[formats.indexOf("YYYY")] : formats[formats.indexOf("YY")];
					return format || "YYYY";
				}

				function getValue(currentValue, keyCode, position){
					var value = currentValue;

					if(keyCode === BACKSPACE || keyCode === DELETE){
						var newPosition = keyCode === BACKSPACE ? position - 1 : position;

						if(currentValue[newPosition] !== separator){
							value = replaceAt(value, newPosition, '_');
						}
					}

					return value;
				}

				function updatePosition(evt){
					var position;
					var pos = getPos();
					var keyCode = evt.keyCode;

					if(keyCode === BACKSPACE){
						position = pos - 1;
					} else if(keyCode === DELETE){
						position = pos;
					} else {
						position = getSelectionStart();
					}
					setCursorPosition(position);
					return position;
				}

				function isToPreventDefault(evt){
					return evt.keyCode && (evt.key.length < 2 ||
						evt.keyCode === BACKSPACE ||
						evt.keyCode === DELETE ||
						evt.keyCode === CONTROL);
				}

				function replaceAt(string, index, replacement) {
					return string.substr(0, index) + replacement + string.substr(index + replacement.length);
				}

				$scope.fillCurrentRow = $scope.fillCurrentRow || function(element){
					$scope.fieldRow = element.value;
				};

				$scope.dateFormat = dateFormat;
				$scope.displayFormat = displayFormat;

				$element.on('focus', function(){
					if(Util.isEmptyOrBlank(this.value) && $scope.setEmptyValue){
						$scope.setEmptyValue();
					}
					if(!valueIsFullfilled(this.value) || Util.isEmptyOrBlank(this.value)){
						var position = this.value.indexOf('_');
						$timeout(function(){
							setCursorPosition(position);
						}, 0, false);
					}
				});

				$element.on('blur', function(){
					if(valueIsFullfilled(this.value)){
						if(moment(this.value, displayFormat).isValid()){
							$scope.fillCurrentRow(this);
							templateManager.updateTemplate();

							if($scope.field.validate($scope.field.value()) && $scope.field.change) {
								$scope.field.change();
							}
						} else {
							if($scope.field.intervalOnInvalidInput){
								$scope.field.intervalOnInvalidInput();
							}

							$scope.field.clearValue($scope.field, this.id);
						}
					} else {
						$scope.field.clearValue($scope.field, this.id);
					}
				});

				$element.on('keydown', function(evt){
					var key = evt.key;
					var keyCode = evt.keyCode;
					var values;
					var position;

					if(isToPreventDefault(evt)){
						evt.preventDefault();
					}

					if(!isNaN(key) && key !== " "){
						var oldValue = this.value;
						position = this.selectionStart;

						if(oldValue[position] === separator) position++;

						var tempValue = replaceAt(oldValue, position, key);
						values = tempValue.split(separator);
						this.value = formats
							.map(function(format, index){
								var match = values[index].match(getRegExpForFormat(format));
								var yearFormat = getYearFormat(formats);
								if(!match){
									values[index] = replaceAt(values[index], 1 - values[index].indexOf(key), "0");
								}
								if(values[index] == 0){
									values[index] = format === "YYYY" ? "0001" : "01";
								}

								if(valueIsFullfilled(values[index])){
									if(format === "MM" && values[index] > 12){
										values[index] = "12";
									}

									var year = values[formats.indexOf(yearFormat)];
									var month = values[formats.indexOf("MM")];
									var daysInMonth = getMaxDayOfMonth(month, year, yearFormat);
									if(format === "DD" && values[index] > daysInMonth){
										values[index] = daysInMonth;
									}
								}

								return values[index];
							})
							.join(separator);

						if(this.value[position] !== "_"){
							position++;
						}
						setCursorPosition(position);
					} else if(keyCode === BACKSPACE || keyCode === DELETE){
						this.pos = this.selectionStart;
					}

					var value = getValue(this.value, keyCode, this.pos);
					values = value.split(separator);

					formats.forEach(function(format, index){
						var value = values[index] || "";
						var valueSize =  value.length;
						var formatSize = format.length;

						if(valueSize < formatSize){
							values[index] = format.substring(0, formatSize - valueSize).replace(/[DMY]/gi, '_') + value;
						} else if(valueSize > formatSize){
							values[index] = value.slice(0, formatSize - valueSize);
						}
					});

					this.value = values.join(separator);
					position = updatePosition(evt);

					if(displayFormat.length == position && keyCode !== ARROW_UP && keyCode !== ARROW_LEFT){
						dateCompleted($element);
					}
				});
			}
		};
	});

})();